@@ -16,7 +16,9 @@
 #include "torrent/object_stream.h"
 #include "torrent/poll.h"
 #include "torrent/throttle.h"
+#include "torrent/net/fd.h"
 #include "torrent/net/socket_address.h"
+#include "torrent/net/network_config.h"
 #include "torrent/utils/log.h"
 #include "tracker/tracker_dht.h"
 
@@ -109,7 +111,7 @@ DhtServer::start(int port) {
     LT_LOG_THIS("starting server : %s", sap_pretty_str(bind_address).c_str());
 
     // Figure out how to bind to both inet and inet6.
-    if (!get_fd().bind_sa(bind_address.get()))
+    if (!fd_bind(get_fd().get_fd(), bind_address.get()))
       throw resource_error("could not bind datagram socket : " + std::string(strerror(errno)));
 
   } catch (const torrent::base_error&) {
@@ -542,7 +544,7 @@ DhtServer::create_query(transaction_itr itr, int tID, [[maybe_unused]] const soc
     case DhtTransaction::DHT_ANNOUNCE_PEER:
       query[key_a_infoHash] = transaction->as_announce_peer()->info_hash_raw_string();
       query[key_a_token] = transaction->as_announce_peer()->token();
-      query[key_a_port] = manager->connection_manager()->listen_port();
+      query[key_a_port] = config::network_config()->listen_port();
       break;
   }
 
