@@ -8,7 +8,6 @@
 #include "dht/dht_transaction.h"
 #include "net/socket_datagram.h"
 #include "net/throttle_node.h"
-#include "rak/socket_address.h"
 #include "torrent/hash_string.h"
 #include "torrent/object_raw_bencode.h"
 #include "torrent/utils/scheduler.h"
@@ -45,7 +44,7 @@ public:
   void                reset_statistics();
 
   // Contact a node to see if it replies. Set id=0 if unknown.
-  void                ping(const HashString& id, const rak::socket_address* sa);
+  void                ping(const HashString& id, const sockaddr* sa);
 
   // Do a find_node search with the given contacts as starting point for the
   // search.
@@ -83,15 +82,14 @@ private:
     char                 _id[20];
     SocketAddressCompact _addr;
 
-    HashString&          id()          { return *HashString::cast_from(_id); }
-    rak::socket_address  address()     { return rak::socket_address(_addr); }
+    HashString&          id()            { return *HashString::cast_from(_id); }
   };
 
-  using packet_queue   = std::deque<DhtTransactionPacket*>;
+  using packet_queue   = std::deque<std::shared_ptr<DhtTransactionPacket>>;
   using node_info_list = std::list<compact_node_info>;
 
   // Pending transactions.
-  using transaction_map = std::map<DhtTransaction::key_type, DhtTransaction*>;
+  using transaction_map = std::map<DhtTransaction::key_type, std::shared_ptr<DhtTransaction>>;
   using transaction_itr = transaction_map::iterator;
 
   // DHT transaction names for given transaction type.
@@ -109,26 +107,27 @@ private:
 
   void                start_write();
 
-  void                process_query(const HashString& id, const rak::socket_address* sa, const DhtMessage& req);
-  void                process_response(const HashString& id, const rak::socket_address* sa, const DhtMessage& req);
-  void                process_error(const rak::socket_address* sa, const DhtMessage& error);
+  void                process_query(const HashString& id, const sockaddr* sa, const DhtMessage& req);
+  void                process_response(const HashString& id, const sockaddr* sa, const DhtMessage& req);
+  void                process_error(const sockaddr* sa, const DhtMessage& error);
 
   void                parse_find_node_reply(DhtTransactionSearch* t, raw_string nodes);
   void                parse_get_peers_reply(DhtTransactionGetPeers* t, const DhtMessage& res);
 
   void                find_node_next(DhtTransactionSearch* t);
 
-  void                add_packet(DhtTransactionPacket* packet, int priority);
+  void                add_packet(std::shared_ptr<DhtTransactionPacket> packet, int priority);
   void                drop_packet(DhtTransactionPacket* packet);
-  void                create_query(transaction_itr itr, int tID, const rak::socket_address* sa, int priority);
-  void                create_response(const DhtMessage& req, const rak::socket_address* sa, DhtMessage& reply);
-  void                create_error(const DhtMessage& req, const rak::socket_address* sa, int num, const char* msg);
+
+  void                create_query(transaction_itr itr, int tID, const sockaddr* sa, int priority);
+  void                create_response(const DhtMessage& req, const sockaddr* sa, DhtMessage& reply);
+  void                create_error(const DhtMessage& req, const sockaddr* sa, int num, const char* msg);
 
   void                create_find_node_response(const DhtMessage& arg, DhtMessage& reply);
-  void                create_get_peers_response(const DhtMessage& arg, const rak::socket_address* sa, DhtMessage& reply);
-  void                create_announce_peer_response(const DhtMessage& arg, const rak::socket_address* sa, DhtMessage& reply);
+  void                create_get_peers_response(const DhtMessage& arg, const sockaddr* sa, DhtMessage& reply);
+  void                create_announce_peer_response(const DhtMessage& arg, const sockaddr* sa, DhtMessage& reply);
 
-  int                 add_transaction(DhtTransaction* t, int priority);
+  int                 add_transaction(std::shared_ptr<DhtTransaction> transaction, int priority);
 
   // This returns the iterator after the given one or end()
   transaction_itr     failed_transaction(transaction_itr itr, bool quick);
@@ -160,6 +159,6 @@ private:
   bool                m_networkUp{false};
 };
 
-}
+} // namespace torrent
 
 #endif
