@@ -25,11 +25,11 @@ public:
 private:
   void                parse_name(const Object& b);
   void                parse_info(const Object& b);
-  void                parse_magnet_uri(Object& b, const std::string& uri);
+  static void         parse_magnet_uri(Object& b, const std::string& uri);
 
   void                add_tracker_group(const Object& b);
   void                add_tracker_single(const Object& b, int group);
-  void                add_dht_node(const Object& b);
+  static void         add_dht_node(const Object& b);
 
   static bool         is_valid_path_element(const Object& b);
   static bool         is_invalid_path_element(const Object& b) { return !is_valid_path_element(b); }
@@ -37,7 +37,7 @@ private:
   void                parse_single_file(const Object& b, uint32_t chunkSize);
   void                parse_multi_files(const Object& b, uint32_t chunkSize);
 
-  inline Path         create_path(const Object::list_type& plist, const std::string& enc);
+  static Path         create_path(const Object::list_type& plist, const std::string& enc);
   inline Path         choose_path(std::list<Path>* pathList);
 
   DownloadWrapper*    m_download{};
