@@ -5,19 +5,18 @@
 #include "data/chunk_list.h"
 #include "data/hash_queue.h"
 #include "data/hash_torrent.h"
-#include "download/available_list.h"
 #include "download/chunk_selector.h"
 #include "protocol/handshake_manager.h"
 #include "protocol/peer_connection_base.h"
 #include "torrent/data/file.h"
 #include "torrent/data/file_list.h"
 #include "torrent/data/file_manager.h"
+#include "torrent/download_info.h"
 #include "torrent/peer/connection_list.h"
 #include "torrent/peer/peer.h"
 #include "torrent/tracker/manager.h"
 #include "torrent/utils/log.h"
 #include "tracker/thread_tracker.h"
-#include "tracker/tracker_controller.h"
 #include "tracker/tracker_list.h"
 #include "utils/functional.h"
 #include "utils/sha1.h"
@@ -371,4 +370,4 @@ DownloadWrapper::finished_download() {
   info()->mutable_down_rate()->reset_rate();
 }
 
-}
+} // namespace torrent
