@@ -11,8 +11,6 @@ namespace torrent {
 
 class Listen : public SocketBase {
 public:
-  using slot_connection = std::function<void(SocketFd, const rak::socket_address&)>;
-
   ~Listen() override { close(); }
 
   bool                open(uint16_t first, uint16_t last, int backlog, const sockaddr* bind_address);
@@ -22,7 +20,7 @@ public:
 
   uint16_t            port() const { return m_port; }
 
-  slot_connection&    slot_accepted() { return m_slot_accepted; }
+  auto&               slot_accepted() { return m_slot_accepted; }
 
   void                event_read() override;
   void                event_write() override;
@@ -31,7 +29,7 @@ public:
 private:
   uint64_t            m_port{0};
 
-  slot_connection     m_slot_accepted;
+  std::function<void(int, const sockaddr*)> m_slot_accepted;
 };
 
 } // namespace torrent
