@@ -2,9 +2,6 @@
 
 #include "socket_listen.h"
 
-#include <algorithm>
-
-#include "torrent/connection_manager.h"
 #include "torrent/exceptions.h"
 #include "torrent/utils/log.h"
 #include "torrent/utils/random.h"
@@ -23,7 +20,7 @@ socket_listen::set_backlog(int backlog) {
 }
 
 bool
-socket_listen::open(sa_unique_ptr&& sap, uint16_t first_port, uint16_t last_port, uint16_t start_port, fd_flags open_flags) {
+socket_listen::open(sa_unique_ptr sap, uint16_t first_port, uint16_t last_port, uint16_t start_port, fd_flags open_flags) {
   if (is_open())
     throw internal_error("socket_listen::open: already open");
 
@@ -131,4 +128,4 @@ socket_listen::m_open_port(int fd, sa_unique_ptr& sap, uint16_t port) {
   return true;
 }
 
-}
+} // namespace torrent
