@@ -47,6 +47,8 @@ namespace torrent {
 
 class SocketStream : public SocketBase {
 public:
+  ~SocketStream() override;
+
   int                 read_stream(void* buf, uint32_t length);
   int                 write_stream(const void* buf, uint32_t length);
 
@@ -94,6 +96,6 @@ SocketStream::write_stream(const void* buf, uint32_t length) {
   return ::send(m_fileDesc, buf, length, 0);
 }
 
-}
+} // namespace torrent
 
 #endif
