@@ -6,14 +6,12 @@
 #include <netinet/in.h>
 #include <sys/socket.h>
 
-#include "manager.h"
 #include "net/udns/udns.h"
 #include "torrent/common.h"
 #include "torrent/exceptions.h"
-#include "torrent/poll.h"
 #include "torrent/net/socket_address.h"
+#include "torrent/poll.h"
 #include "torrent/utils/log.h"
-#include "torrent/utils/thread.h"
 
 #define LT_LOG(log_fmt, ...)                                \
   lt_log_print_subsystem(LOG_NET_DNS, "dns", log_fmt, __VA_ARGS__);
@@ -98,7 +96,7 @@ UdnsResolver::resolve(void* requester, const std::string& hostname, int family,
       query->error_sin = EAI_NONAME;
 
       auto lock = std::scoped_lock(m_mutex);
-      m_malformed_queries.insert({requester, std::move(query)});
+      m_malformed_queries.emplace(requester, std::move(query));
 
       return;
     }
@@ -123,7 +121,7 @@ UdnsResolver::resolve(void* requester, const std::string& hostname, int family,
       query->error_sin = EAI_NONAME;
 
       auto lock = std::scoped_lock(m_mutex);
-      m_malformed_queries.insert({requester, std::move(query)});
+      m_malformed_queries.emplace(requester, std::move(query));
 
       return;
     }
@@ -132,7 +130,7 @@ UdnsResolver::resolve(void* requester, const std::string& hostname, int family,
   LT_LOG("resolving : requester:%p name:%s family:%d", requester, hostname.c_str(), family);
 
   auto lock = std::scoped_lock(m_mutex);
-  m_queries.insert({requester, std::move(query)});
+  m_queries.emplace(requester, std::move(query));
 }
 
 bool
