@@ -3,7 +3,7 @@
 #include "handshake.h"
 
 #include <cassert>
-#include <stdio.h>
+#include <cstdio>
 
 #include "manager.h"
 #include "download/download_main.h"
@@ -11,14 +11,16 @@
 #include "protocol/extensions.h"
 #include "protocol/handshake_manager.h"
 #include "torrent/download_info.h"
-#include "torrent/exceptions.h"
 #include "torrent/error.h"
+#include "torrent/exceptions.h"
 #include "torrent/poll.h"
 #include "torrent/throttle.h"
+#include "torrent/net/fd.h"
+#include "torrent/net/network_config.h"
 #include "torrent/tracker/dht_controller.h"
 #include "torrent/utils/log.h"
-#include "torrent/utils/thread.h"
 #include "utils/diffie_hellman.h"
+#include "utils/sha1.h"
 
 #define LT_LOG(log_fmt, ...)                                            \
   lt_log_print(LOG_CONNECTION_HANDSHAKE, "handshake->%s: " log_fmt,     \
@@ -31,27 +33,28 @@
 #define LT_LOG_EXTRA_DEBUG_SA(sa, log_fmt, ...)
 #endif
 
-namespace torrent {
+namespace {
+class handshake_succeeded : public torrent::network_error {
+};
 
-class handshake_error : public network_error {
+class handshake_error : public torrent::network_error {
 public:
   handshake_error(int type, int error) : m_type(type), m_error(error) {}
 
-  virtual const char* what() const throw()  { return "Handshake error"; }
-  virtual int         type() const throw()  { return m_type; }
-  virtual int         error() const throw() { return m_error; }
+  const char* what() const noexcept override  { return "Handshake error"; }
+  int         type() const noexcept           { return m_type; }
+  int         error() const noexcept          { return m_error; }
 
 private:
   int     m_type;
   int     m_error;
 };
 
-class handshake_succeeded : public network_error {
-public:
-  using network_error::network_error;
-};
+} // namespace
 
-Handshake::Handshake(SocketFd fd, HandshakeManager* m, int encryptionOptions) :
+namespace torrent {
+
+Handshake::Handshake(int fd, HandshakeManager* m, int encryptionOptions) :
   m_manager(m),
 
   // Use global throttles until we know which download it is.
@@ -59,9 +62,9 @@ Handshake::Handshake(SocketFd fd, HandshakeManager* m, int encryptionOptions) :
   m_downloadThrottle(manager->download_throttle()->throttle_list()),
 
   m_encryption(encryptionOptions),
-  m_extensions(m->default_extensions()) {
+  m_extensions(torrent::HandshakeManager::default_extensions()) {
 
-  set_fd(fd);
+  set_fd(SocketFd(fd));
 
   m_readBuffer.reset();
   m_writeBuffer.reset();
@@ -81,7 +84,7 @@ Handshake::initialize_incoming(const sockaddr* sa) {
   m_incoming = true;
   m_address = sa_copy(sa);
 
-  if (m_encryption.options() & (ConnectionManager::encryption_allow_incoming | ConnectionManager::encryption_require))
+  if (m_encryption.options() & (net::NetworkConfig::encryption_allow_incoming | net::NetworkConfig::encryption_require))
     m_state = READ_ENC_KEY;
   else
     m_state = READ_INFO;
@@ -164,12 +167,12 @@ Handshake::destroy_connection() {
 
 int
 Handshake::retry_options() {
-  uint32_t options = m_encryption.options() & ~ConnectionManager::encryption_enable_retry;
+  uint32_t options = m_encryption.options() & ~net::NetworkConfig::encryption_enable_retry;
 
   if (m_encryption.retry() == HandshakeEncryption::RETRY_PLAIN)
-    options &= ~ConnectionManager::encryption_try_outgoing;
+    options &= ~net::NetworkConfig::encryption_try_outgoing;
   else if (m_encryption.retry() == HandshakeEncryption::RETRY_ENCRYPTED)
-    options |= ConnectionManager::encryption_try_outgoing;
+    options |= net::NetworkConfig::encryption_try_outgoing;
   else
     throw internal_error("Invalid retry type.");
 
@@ -224,7 +227,7 @@ Handshake::read_encryption_key() {
 
     if (m_readBuffer.peek_8() == 19 && std::memcmp(m_readBuffer.position() + 1, m_protocol, 19) == 0) {
       // got unencrypted BT handshake
-      if (m_encryption.options() & ConnectionManager::encryption_require)
+      if (m_encryption.options() & net::NetworkConfig::encryption_require)
         throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_unencrypted_rejected);
 
       m_state = READ_INFO;
@@ -364,10 +367,10 @@ Handshake::read_encryption_negotiation() {
 
   // choose one of the offered encryptions, or check the chosen one is valid
   if (m_incoming) {
-    if ((m_encryption.options() & ConnectionManager::encryption_prefer_plaintext) && m_encryption.has_crypto_plain()) {
+    if ((m_encryption.options() & net::NetworkConfig::encryption_prefer_plaintext) && m_encryption.has_crypto_plain()) {
       m_encryption.set_crypto(HandshakeEncryption::crypto_plain);
 
-    } else if ((m_encryption.options() & ConnectionManager::encryption_require_RC4) && !m_encryption.has_crypto_rc4()) {
+    } else if ((m_encryption.options() & net::NetworkConfig::encryption_require_RC4) && !m_encryption.has_crypto_rc4()) {
       throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_unencrypted_rejected);
 
     } else if (m_encryption.has_crypto_rc4()) {
@@ -389,7 +392,7 @@ Handshake::read_encryption_negotiation() {
     if (m_encryption.crypto() != HandshakeEncryption::crypto_rc4 && m_encryption.crypto() != HandshakeEncryption::crypto_plain)
       throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_encryption);
 
-    if ((m_encryption.options() & ConnectionManager::encryption_require_RC4) && (m_encryption.crypto() != HandshakeEncryption::crypto_rc4))
+    if ((m_encryption.options() & net::NetworkConfig::encryption_require_RC4) && (m_encryption.crypto() != HandshakeEncryption::crypto_rc4))
       throw handshake_error(ConnectionManager::handshake_failed, e_handshake_invalid_encryption);
   }
 
@@ -671,6 +674,7 @@ restart:
       if (m_state != READ_ENC_SYNC)
         goto restart;
 
+      [[fallthrough]];
     case READ_ENC_SYNC:
       if (!read_encryption_sync())
         break;
@@ -678,10 +682,12 @@ restart:
       if (m_state != READ_ENC_SKEY)
         goto restart;
 
+      [[fallthrough]];
     case READ_ENC_SKEY:
       if (!read_encryption_skey())
         break;
 
+      [[fallthrough]];
     case READ_ENC_NEGOT:
       if (!read_encryption_negotiation())
         break;
@@ -689,6 +695,7 @@ restart:
       if (m_state != READ_ENC_PAD)
         goto restart;
 
+      [[fallthrough]];
     case READ_ENC_PAD:
       if (m_readPos) {
         LT_LOG_EXTRA_DEBUG_SA(m_address, "event_read : READ_ENC_PAD : m_readPos:%" PRIu32, m_readPos)
@@ -708,6 +715,7 @@ restart:
       if (m_state != READ_ENC_IA)
         goto restart;
 
+      [[fallthrough]];
     case READ_ENC_IA:
       LT_LOG_EXTRA_DEBUG_SA(m_address, "event_read : READ_ENC_IA", 0)
 
@@ -724,6 +732,7 @@ restart:
 
       m_state = READ_INFO;
 
+      [[fallthrough]];
     case READ_INFO:
       if (!read_info())
         break;
@@ -731,6 +740,7 @@ restart:
       if (m_state != READ_PEER)
         goto restart;
 
+      [[fallthrough]];
     case READ_PEER:
       if (!read_peer())
         break;
@@ -739,6 +749,7 @@ restart:
       if (m_state != READ_MESSAGE)
         goto restart;
 
+      [[fallthrough]];
     case READ_MESSAGE:
     case POST_HANDSHAKE:
       // For meta-downloads, we aren't interested in the bitfield or
@@ -809,6 +820,7 @@ restart:
         break;
       }
 
+      [[fallthrough]];
     case READ_BITFIELD:
     case READ_EXT:
     case READ_PORT:
@@ -839,15 +851,15 @@ restart:
       return event_write();
     }
 
-  } catch (handshake_succeeded& e) {
-    m_manager->receive_succeeded(this);
+} catch (const handshake_succeeded&) {
+  m_manager->receive_succeeded(this);
 
-  } catch (handshake_error& e) {
-    m_manager->receive_failed(this, e.type(), e.error());
+} catch (const handshake_error& e) {
+  m_manager->receive_failed(this, e.type(), e.error());
 
-  } catch (network_error& e) {
-    m_manager->receive_failed(this, ConnectionManager::handshake_failed, e_handshake_network_read_error);
-  }
+} catch (const network_error&) {
+  m_manager->receive_failed(this, ConnectionManager::handshake_failed, e_handshake_network_read_error);
+}
 }
 
 bool
@@ -880,22 +892,28 @@ Handshake::validate_download() {
 
 void
 Handshake::event_write() {
+  int socket_error;
+
   try {
 
     switch (m_state) {
     case CONNECTING:
-      if (get_fd().get_error())
+      if (!fd_get_socket_error(m_fileDesc, &socket_error))
+        throw internal_error("Handshake::event_write() fd_get_socket_error failed : " + std::string(strerror(errno)));
+
+      if (socket_error != 0)
         throw handshake_error(ConnectionManager::handshake_failed, e_handshake_network_unreachable);
 
       this_thread::poll()->insert_read(this);
 
-      if (m_encryption.options() & ConnectionManager::encryption_use_proxy) {
+      if (m_encryption.options() & net::NetworkConfig::encryption_use_proxy) {
         prepare_proxy_connect();
 
         m_state = PROXY_CONNECT;
         break;
       }
 
+      [[fallthrough]];
     case PROXY_DONE:
       // If there's any bytes remaining, it means we got a reply from
       // the other side before our proxy connect command was finished
@@ -905,11 +923,11 @@ Handshake::event_write() {
 
       m_writeBuffer.reset();
 
-      if (m_encryption.options() & (ConnectionManager::encryption_try_outgoing | ConnectionManager::encryption_require)) {
+      if (m_encryption.options() & (net::NetworkConfig::encryption_try_outgoing | net::NetworkConfig::encryption_require)) {
         prepare_key_plus_pad();
 
         // if connection fails, peer probably closed it because it was encrypted, so retry encrypted if enabled
-        if (!(m_encryption.options() & ConnectionManager::encryption_require))
+        if (!(m_encryption.options() & net::NetworkConfig::encryption_require))
           m_encryption.set_retry(HandshakeEncryption::RETRY_PLAIN);
 
         m_state = READ_ENC_KEY;
@@ -949,13 +967,13 @@ Handshake::event_write() {
         this_thread::poll()->remove_write(this);
     }
 
-  } catch (handshake_succeeded& e) {
+  } catch (const handshake_succeeded&) {
     m_manager->receive_succeeded(this);
 
-  } catch (handshake_error& e) {
+  } catch (const handshake_error& e) {
     m_manager->receive_failed(this, e.type(), e.error());
 
-  } catch (network_error& e) {
+  } catch (const network_error&) {
     m_manager->receive_failed(this, ConnectionManager::handshake_failed, e_handshake_network_write_error);
   }
 }
@@ -1009,7 +1027,7 @@ Handshake::prepare_enc_negotiation() {
   HandshakeEncryption::copy_vc(m_writeBuffer.end());
   m_writeBuffer.move_end(HandshakeEncryption::vc_length);
 
-  if (m_encryption.options() & ConnectionManager::encryption_require_RC4)
+  if (m_encryption.options() & net::NetworkConfig::encryption_require_RC4)
     m_writeBuffer.write_32(HandshakeEncryption::crypto_rc4);
   else
     m_writeBuffer.write_32(HandshakeEncryption::crypto_plain | HandshakeEncryption::crypto_rc4);
@@ -1056,7 +1074,7 @@ Handshake::prepare_peer_info() {
     if (m_peerInfo == NULL)
       throw handshake_error(ConnectionManager::handshake_failed, e_handshake_no_peer_info);
 
-    if (m_peerInfo->failed_counter() > m_manager->max_failed)
+    if (m_peerInfo->failed_counter() > torrent::HandshakeManager::max_failed)
       throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_toomanyfailed);
 
     m_peerInfo->set_flags(PeerInfo::flag_handshake);
@@ -1212,4 +1230,4 @@ Handshake::event_error() {
   m_manager->receive_failed(this, ConnectionManager::handshake_failed, e_handshake_network_socket_error);
 }
 
-}
+} // namespace torrent
