@@ -3,19 +3,19 @@
 #include <cstring>
 #include <sstream>
 
-#include "manager.h"
-#include "data/chunk_list_node.h"
 #include "download/chunk_selector.h"
 #include "download/chunk_statistics.h"
 #include "download/download_main.h"
-#include "rak/string_manip.h"
-#include "torrent/download_info.h"
+#include "manager.h"
 #include "torrent/download/choke_group.h"
 #include "torrent/download/choke_queue.h"
+#include "torrent/download_info.h"
+#include "torrent/net/socket_address.h"
 #include "torrent/peer/connection_list.h"
 #include "torrent/peer/peer_info.h"
 #include "torrent/tracker/dht_controller.h"
 #include "torrent/utils/log.h"
+#include "torrent/utils/uri_parser.h"
 
 #include "extensions.h"
 #include "initial_seed.h"
@@ -28,6 +28,7 @@
 
 namespace torrent {
 
+#if 0
 template<Download::ConnectionType type>
 PeerConnection<type>::~PeerConnection() {
 //   if (m_download != NULL && m_down->get_state() != ProtocolRead::READ_BITFIELD)
@@ -35,6 +36,10 @@ PeerConnection<type>::~PeerConnection() {
 
 //   priority_queue_erase(&taskScheduler, &m_taskSendChoke);
 }
+#else
+template<Download::ConnectionType type>
+PeerConnection<type>::~PeerConnection() = default;
+#endif
 
 template<Download::ConnectionType type>
 void
@@ -428,26 +433,24 @@ PeerConnection<type>::event_read() {
 
   // Exception handlers:
 
-  } catch (close_connection& e) {
+  } catch (const close_connection&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (blocked_connection& e) {
+  } catch (const blocked_connection&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (network_error& e) {
-    LT_LOG_NETWORK_ERRORS("%s network read error: %s",
-                          rak::socket_address::cast_from(m_peerInfo->socket_address())->address_str().c_str(),
-                          e.what());
+  } catch (const network_error& e) {
+    LT_LOG_NETWORK_ERRORS("network read error: %s : %s", sa_pretty_str(m_peerInfo->socket_address()).c_str(), e.what());
     m_download->connection_list()->erase(this, 0);
 
-  } catch (storage_error& e) {
+  } catch (const storage_error& e) {
     LT_LOG_NETWORK_ERRORS("storage read error: %s", e.what());
     m_download->connection_list()->erase(this, 0);
 
-  } catch (base_error& e) {
+  } catch (const base_error& e) {
     std::stringstream s;
     s << "Connection read fd(" << get_fd().get_fd() << ',' << m_down->get_state() << ',' << m_down->last_command() << ") \"" << e.what() << '"';
-    s << " '" << rak::copy_escape_html((char*)m_down->buffer()->begin(), (char*)m_down->buffer()->position()) << "'";
+    s << " '" << utils::uri_escape_html(reinterpret_cast<char*>(m_down->buffer()->begin()), reinterpret_cast<char*>(m_down->buffer()->position())) << "'";
 
     throw internal_error(s.str());
   }
@@ -468,12 +471,11 @@ PeerConnection<type>::fill_write_buffer() {
       up_chunk_release();
       m_peerChunks.upload_queue()->clear();
 
-      if (m_encryptBuffer != NULL) {
+      if (m_encryptBuffer != nullptr) {
         if (m_encryptBuffer->remaining())
           throw internal_error("Deleting encryptBuffer with encrypted data remaining.");
 
-        delete m_encryptBuffer;
-        m_encryptBuffer = NULL;
+        m_encryptBuffer = nullptr;
       }
 
     } else {
@@ -566,6 +568,7 @@ PeerConnection<type>::event_write() {
 
         m_up->set_state(ProtocolWrite::MSG);
 
+	[[fallthrough]];
       case ProtocolWrite::MSG:
         if (!m_up->buffer()->consume(m_up->throttle()->node_used_unthrottled(write_stream_throws(m_up->buffer()->position(), m_up->buffer()->remaining()))))
           return;
@@ -590,6 +593,7 @@ PeerConnection<type>::event_write() {
           break;
         }
 
+	[[fallthrough]];
       case ProtocolWrite::WRITE_PIECE:
         if (!up_chunk())
           return;
@@ -610,23 +614,21 @@ PeerConnection<type>::event_write() {
 
     } while (true);
 
-  } catch (close_connection& e) {
+  } catch (const close_connection&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (blocked_connection& e) {
+  } catch (const blocked_connection&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (network_error& e) {
-    LT_LOG_NETWORK_ERRORS("%s write error: %s",
-                          rak::socket_address::cast_from(m_peerInfo->socket_address())->address_str().c_str(),
-                          e.what());
+  } catch (const network_error& e) {
+    LT_LOG_NETWORK_ERRORS("network write error: %s : %s", sa_pretty_str(m_peerInfo->socket_address()).c_str(), e.what());
     m_download->connection_list()->erase(this, 0);
 
-  } catch (storage_error& e) {
+  } catch (const storage_error& e) {
     LT_LOG_STORAGE_ERRORS("write error: %s", e.what());
     m_download->connection_list()->erase(this, 0);
 
-  } catch (base_error& e) {
+  } catch (const base_error& e) {
     std::stringstream s;
     s << "Connection write fd(" << get_fd().get_fd() << ',' << m_up->get_state() << ',' << m_up->last_command() << ") \"" << e.what() << '"';
 
@@ -651,7 +653,7 @@ PeerConnection<type>::read_have_chunk(uint32_t index) {
   // Disconnect seeds when we are seeding (but not for initial seeding
   // so that we keep accurate chunk statistics until that is done).
   if (m_peerChunks.bitfield()->is_all_set()) {
-    if (type == Download::CONNECTION_SEED || 
+    if (type == Download::CONNECTION_SEED ||
         (type != Download::CONNECTION_INITIAL_SEED && m_download->file_list()->is_done()))
       throw close_connection();
 
@@ -673,7 +675,7 @@ PeerConnection<type>::read_have_chunk(uint32_t index) {
     if (m_download->chunk_selector()->received_have_chunk(&m_peerChunks, index)) {
       m_sendInterested = !m_downInterested;
       m_downInterested = true;
-      
+
       // Ensure we get inserted into the choke manager queue in case
       // the peer keeps us unchoked even though we've said we're not
       // interested.
@@ -741,4 +743,4 @@ template class PeerConnection<Download::CONNECTION_LEECH>;
 template class PeerConnection<Download::CONNECTION_SEED>;
 template class PeerConnection<Download::CONNECTION_INITIAL_SEED>;
 
-}
+} // namespace torrent
