@@ -5,14 +5,13 @@
 #include <cstring>
 #include <sstream>
 
-#include "manager.h"
-#include "data/chunk_list_node.h"
 #include "download/chunk_selector.h"
 #include "download/chunk_statistics.h"
 #include "download/download_main.h"
+#include "manager.h"
 #include "protocol/extensions.h"
-#include "torrent/download_info.h"
 #include "torrent/download/choke_queue.h"
+#include "torrent/download_info.h"
 #include "torrent/peer/connection_list.h"
 #include "torrent/peer/peer_info.h"
 #include "torrent/tracker/dht_controller.h"
@@ -254,20 +253,20 @@ PeerConnectionMetadata::event_read() {
 
   // Exception handlers:
 
-  } catch (close_connection& e) {
+  } catch (const close_connection&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (blocked_connection& e) {
+  } catch (const blocked_connection&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (network_error& e) {
+  } catch (const network_error&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (storage_error& e) {
+  } catch (const storage_error& e) {
     LT_LOG_STORAGE_ERRORS("read error: %s", e.what());
     m_download->connection_list()->erase(this, 0);
 
-  } catch (base_error& e) {
+  } catch (const base_error& e) {
     std::stringstream s;
     s << "Connection read fd(" << get_fd().get_fd() << ',' << m_down->get_state() << ',' << m_down->last_command() << ") \"" << e.what() << '"';
 
@@ -313,6 +312,7 @@ PeerConnectionMetadata::event_write() {
 
         m_up->set_state(ProtocolWrite::MSG);
 
+	[[fallthrough]];
       case ProtocolWrite::MSG:
         if (!m_up->buffer()->consume(m_up->throttle()->node_used_unthrottled(write_stream_throws(m_up->buffer()->position(),
                                                                                                  m_up->buffer()->remaining()))))
@@ -327,6 +327,7 @@ PeerConnectionMetadata::event_write() {
 
         m_up->set_state(ProtocolWrite::WRITE_EXTENSION);
 
+	[[fallthrough]];
       case ProtocolWrite::WRITE_EXTENSION:
         if (!up_extension())
           return;
@@ -340,20 +341,20 @@ PeerConnectionMetadata::event_write() {
 
     } while (true);
 
-  } catch (close_connection& e) {
+  } catch (const close_connection&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (blocked_connection& e) {
+  } catch (const blocked_connection&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (network_error& e) {
+  } catch (const network_error&) {
     m_download->connection_list()->erase(this, 0);
 
-  } catch (storage_error& e) {
+  } catch (const storage_error& e) {
     LT_LOG_STORAGE_ERRORS("read error: %s", e.what());
     m_download->connection_list()->erase(this, 0);
 
-  } catch (base_error& e) {
+  } catch (const base_error& e) {
     std::stringstream s;
     s << "Connection write fd(" << get_fd().get_fd() << ',' << m_up->get_state() << ',' << m_up->last_command() << ") \"" << e.what() << '"';
 
@@ -451,4 +452,4 @@ PeerConnectionMetadata::receive_metadata_piece(uint32_t piece, const char* data,
   down_chunk_finished();
 }
 
-}
+} // namespace torrent
