@@ -3,8 +3,6 @@
 #include "torrent/chunk_manager.h"
 
 #include <cassert>
-#include <sys/types.h>
-#include <sys/time.h>
 #include <sys/resource.h>
 
 #include "data/chunk_list.h"
@@ -13,12 +11,7 @@
 
 namespace torrent {
 
-// 1/5 of the available memory should be enough for the client. If
-// the client really requires alot more memory it should call this
-// itself.
-ChunkManager::ChunkManager() :
-    m_maxMemoryUsage((estimate_max_memory_usage() * 4) / 5) {
-}
+ChunkManager::ChunkManager() = default;
 
 ChunkManager::~ChunkManager() {
   assert(m_memoryUsage == 0 && "ChunkManager::~ChunkManager() m_memoryUsage != 0.");
@@ -176,4 +169,4 @@ ChunkManager::sync_all(int flags, uint64_t target) {
   m_lastFreed = itr - begin();
 }
 
-}
+} // namespace torrent
