@@ -3,6 +3,7 @@
 #include <algorithm>
 #include <arpa/inet.h>
 #include <cstring>
+#include <netdb.h>
 #include <sys/un.h>
 
 #include "torrent/exceptions.h"
@@ -103,12 +104,12 @@ sin_is_broadcast(const sockaddr_in* sa) {
 
 bool
 sa_is_v4mapped(const sockaddr* sa) {
-  return sa != NULL && sa->sa_family == AF_INET6 && sin6_is_v4mapped(reinterpret_cast<const sockaddr_in6*>(sa));
+  return sa != nullptr && sa->sa_family == AF_INET6 && sin6_is_v4mapped(reinterpret_cast<const sockaddr_in6*>(sa));
 }
 
 bool
 sin6_is_v4mapped(const sockaddr_in6* sa) {
-  return sa != NULL && IN6_IS_ADDR_V4MAPPED(&sa->sin6_addr);
+  return sa != nullptr && IN6_IS_ADDR_V4MAPPED(&sa->sin6_addr);
 }
 
 bool
@@ -542,10 +543,68 @@ sin6_equal_addr(const sockaddr_in6* lhs, const sockaddr_in6* rhs) {
   return std::equal(lhs->sin6_addr.s6_addr, lhs->sin6_addr.s6_addr + 16, rhs->sin6_addr.s6_addr);
 }
 
+bool
+sa_less(const sockaddr* lhs, const sockaddr* rhs) {
+  if (lhs->sa_family != AF_INET && lhs->sa_family != AF_INET6)
+    throw internal_error("torrent::sa_less() lhs sockaddr is not inet or inet6");
+
+  if (rhs->sa_family != AF_INET && rhs->sa_family != AF_INET6)
+    throw internal_error("torrent::sa_less() rhs sockaddr is not inet or inet6");
+
+  if (lhs->sa_family != rhs->sa_family)
+    return lhs->sa_family == AF_INET;
+
+  if (lhs->sa_family == AF_INET) {
+    const sockaddr_in* lsin = reinterpret_cast<const sockaddr_in*>(lhs);
+    const sockaddr_in* rsin = reinterpret_cast<const sockaddr_in*>(rhs);
+
+    if (lsin->sin_addr.s_addr != rsin->sin_addr.s_addr)
+      return ntohl(lsin->sin_addr.s_addr) < ntohl(rsin->sin_addr.s_addr);
+
+    return ntohs(lsin->sin_port) < ntohs(rsin->sin_port);
+
+  } else {
+    const sockaddr_in6* lsin6 = reinterpret_cast<const sockaddr_in6*>(lhs);
+    const sockaddr_in6* rsin6 = reinterpret_cast<const sockaddr_in6*>(rhs);
+
+    auto result = std::memcmp(&lsin6->sin6_addr, &rsin6->sin6_addr, sizeof(in6_addr));
+
+    if (result != 0)
+      return result < 0;
+
+    return ntohs(lsin6->sin6_port) < ntohs(rsin6->sin6_port);
+  }
+}
+
+bool
+sa_less_addr(const sockaddr* lhs, const sockaddr* rhs) {
+  if (lhs->sa_family != AF_INET && lhs->sa_family != AF_INET6)
+    throw internal_error("torrent::sa_less_addr() lhs sockaddr is not inet or inet6");
+
+  if (rhs->sa_family != AF_INET && rhs->sa_family != AF_INET6)
+    throw internal_error("torrent::sa_less_addr() rhs sockaddr is not inet or inet6");
+
+  if (lhs->sa_family != rhs->sa_family)
+    return lhs->sa_family == AF_INET;
+
+  if (lhs->sa_family == AF_INET) {
+    const sockaddr_in* lsin = reinterpret_cast<const sockaddr_in*>(lhs);
+    const sockaddr_in* rsin = reinterpret_cast<const sockaddr_in*>(rhs);
+
+    return ntohl(lsin->sin_addr.s_addr) < ntohl(rsin->sin_addr.s_addr);
+
+  } else {
+    const sockaddr_in6* lsin6 = reinterpret_cast<const sockaddr_in6*>(lhs);
+    const sockaddr_in6* rsin6 = reinterpret_cast<const sockaddr_in6*>(rhs);
+
+    return std::memcmp(&lsin6->sin6_addr, &rsin6->sin6_addr, sizeof(in6_addr)) < 0;
+  }
+}
+
 std::string
 sa_addr_str(const sockaddr* sa) {
-  if (sa == NULL)
-    return "unspec";
+  if (sa == nullptr)
+    return "null";
 
   switch (sa->sa_family) {
   case AF_INET:
@@ -583,7 +642,7 @@ sin6_addr_str(const sockaddr_in6* sa) {
 std::string
 sa_pretty_str(const sockaddr* sa) {
   if (sa == nullptr)
-    return "nullptr";
+    return "null";
 
   switch (sa->sa_family) {
   case AF_INET:
@@ -617,25 +676,50 @@ sin6_pretty_str(const sockaddr_in6* sa) {
   return result;
 }
 
-// Deprecated:
+c_sa_shared_ptr
+sa_lookup_address(const std::string& address_str, int family) {
+  if (address_str.empty())
+    return sa_make_unspec();
 
-void
-sa_inet_mapped_inet6(const sockaddr_in* sa, sockaddr_in6* mapped) {
-  uint32_t addr32[4];
-  addr32[0] = 0;
-  addr32[1] = 0;
-  addr32[2] = htonl(0xffff);
-  addr32[3] = sa->sin_addr.s_addr;
+  // don't use rak::, use unix getaddrinfo
 
-  sa_clear_inet6(mapped);
+  addrinfo hints = {};
+  hints.ai_family = family;
+  hints.ai_socktype = SOCK_STREAM;
 
-  mapped->sin6_addr = *reinterpret_cast<in6_addr*>(addr32);
-  mapped->sin6_port = sa->sin_port;
+  addrinfo* res;
+
+  int err = ::getaddrinfo(address_str.c_str(), nullptr, &hints, &res);
+
+  if (err != 0)
+    throw input_error("Could not get address info: " + address_str + ": " + std::string(gai_strerror(err)));
+
+  try {
+    auto sa = sa_copy(res->ai_addr);
+    ::freeaddrinfo(res);
+
+    return sa;
+
+  } catch (input_error& e) {
+    ::freeaddrinfo(res);
+    throw e;
+  }
 }
 
-std::string
-sa_pretty_address_str(const sockaddr* sa) {
-  return sa_pretty_str(sa);
+sa_inet_union
+sa_inet_union_from_sa(const sockaddr* sa) {
+  sa_inet_union su{};
+
+  switch (sa->sa_family) {
+  case AF_INET:
+    su.inet = *reinterpret_cast<const sockaddr_in*>(sa);
+    return su;
+  case AF_INET6:
+    su.inet6 = *reinterpret_cast<const sockaddr_in6*>(sa);
+    return su;
+  default:
+    throw internal_error("torrent::sa_inet_union_from_sa: sockaddr is not inet or inet6");
+  }
 }
 
 } // namespace torrent
