@@ -1,44 +1,9 @@
-// libTorrent - BitTorrent library
-// Copyright (C) 2005-2011, Jari Sundell
-//
-// This program is free software; you can redistribute it and/or modify
-// it under the terms of the GNU General Public License as published by
-// the Free Software Foundation; either version 2 of the License, or
-// (at your option) any later version.
-// 
-// This program is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-// GNU General Public License for more details.
-// 
-// You should have received a copy of the GNU General Public License
-// along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-//
-// In addition, as a special exception, the copyright holders give
-// permission to link the code of portions of this program with the
-// OpenSSL library under certain conditions as described in each
-// individual source file, and distribute linked combinations
-// including the two.
-//
-// You must obey the GNU General Public License in all respects for
-// all of the code used other than OpenSSL.  If you modify file(s)
-// with this exception, you may extend this exception to your version
-// of the file(s), but you are not obligated to do so.  If you do not
-// wish to do so, delete this exception statement from your version.
-// If you delete this exception statement from all source files in the
-// program, then also delete it here.
-//
-// Contact:  Jari Sundell <jaris@ifi.uio.no>
-//
-//           Skomakerveien 33
-//           3185 Skoppum, NORWAY
-
 #ifndef LIBTORRENT_OBJECT_H
 #define LIBTORRENT_OBJECT_H
 
-#include <string>
+#include <limits>
 #include <map>
+#include <string>
 #include <vector>
 #include <torrent/common.h>
 #include <torrent/exceptions.h>
@@ -48,41 +13,41 @@ namespace torrent {
 
 class LIBTORRENT_EXPORT Object {
 public:
-  typedef int64_t                           value_type;
-  typedef std::string                       string_type;
-  typedef std::vector<Object>               list_type;
-  typedef std::map<std::string, Object>     map_type;
-  typedef map_type*                         map_ptr_type;
-  typedef map_type::key_type                key_type;
-  typedef std::pair<std::string, Object*>   dict_key_type;
+  using value_type    = int64_t;
+  using string_type   = std::string;
+  using list_type     = std::vector<Object>;
+  using map_type      = std::map<std::string, Object>;
+  using map_ptr_type  = map_type*;
+  using key_type      = map_type::key_type;
+  using dict_key_type = std::pair<std::string, Object*>;
 
-  typedef list_type::iterator               list_iterator;
-  typedef list_type::const_iterator         list_const_iterator;
-  typedef list_type::reverse_iterator       list_reverse_iterator;
-  typedef list_type::const_reverse_iterator list_const_reverse_iterator;
+  using list_iterator               = list_type::iterator;
+  using list_const_iterator         = list_type::const_iterator;
+  using list_reverse_iterator       = list_type::reverse_iterator;
+  using list_const_reverse_iterator = list_type::const_reverse_iterator;
 
-  typedef map_type::iterator                map_iterator;
-  typedef map_type::const_iterator          map_const_iterator;
-  typedef map_type::reverse_iterator        map_reverse_iterator;
-  typedef map_type::const_reverse_iterator  map_const_reverse_iterator;
+  using map_iterator               = map_type::iterator;
+  using map_const_iterator         = map_type::const_iterator;
+  using map_reverse_iterator       = map_type::reverse_iterator;
+  using map_const_reverse_iterator = map_type::const_reverse_iterator;
 
-  typedef std::pair<map_iterator, bool>     map_insert_type;
+  using map_insert_type = std::pair<map_iterator, bool>;
 
   // Flags in the range of 0xffff0000 may be set by the user, however
   // 0x00ff0000 are reserved for keywords defined by libtorrent.
-  static const uint32_t mask_type     = 0xff;
-  static const uint32_t mask_flags    = ~mask_type;
-  static const uint32_t mask_internal = 0xffff;
-  static const uint32_t mask_public   = ~mask_internal;
+  static constexpr uint32_t mask_type     = 0xff;
+  static constexpr uint32_t mask_flags    = ~mask_type;
+  static constexpr uint32_t mask_internal = 0xffff;
+  static constexpr uint32_t mask_public   = ~mask_internal;
 
-  static const uint32_t flag_unordered    = 0x100;    // bencode dictionary was not sorted
-  static const uint32_t flag_static_data  = 0x010000;  // Object does not change across sessions.
-  static const uint32_t flag_session_data = 0x020000;  // Object changes between sessions.
-  static const uint32_t flag_function     = 0x040000;  // A function object.
-  static const uint32_t flag_function_q1  = 0x080000;  // A quoted function object.
-  static const uint32_t flag_function_q2  = 0x100000;  // A double-quoted function object.
+  static constexpr uint32_t flag_unordered    = 0x100;    // bencode dictionary was not sorted
+  static constexpr uint32_t flag_static_data  = 0x010000;  // Object does not change across sessions.
+  static constexpr uint32_t flag_session_data = 0x020000;  // Object changes between sessions.
+  static constexpr uint32_t flag_function     = 0x040000;  // A function object.
+  static constexpr uint32_t flag_function_q1  = 0x080000;  // A quoted function object.
+  static constexpr uint32_t flag_function_q2  = 0x100000;  // A double-quoted function object.
 
-  static const uint32_t mask_function     = 0x1C0000;  // Mask for function objects.
+  static constexpr uint32_t mask_function     = 0x1C0000;  // Mask for function objects.
 
   enum type_type {
     TYPE_NONE,
@@ -97,7 +62,6 @@ public:
     TYPE_DICT_KEY
   };
 
-  Object()                     : m_flags(TYPE_NONE) {}
   Object(const value_type v)   : m_flags(TYPE_VALUE) { new (&_value()) value_type(v); }
   Object(const char* s)        : m_flags(TYPE_STRING) { new (&_string()) string_type(s); }
   Object(const string_type& s) : m_flags(TYPE_STRING) { new (&_string()) string_type(s); }
@@ -105,9 +69,11 @@ public:
   Object(const raw_string& r)  : m_flags(TYPE_RAW_STRING) { new (&_raw_string()) raw_string(r); }
   Object(const raw_list& r)    : m_flags(TYPE_RAW_LIST) { new (&_raw_list()) raw_list(r); }
   Object(const raw_map& r)     : m_flags(TYPE_RAW_MAP) { new (&_raw_map()) raw_map(r); }
-  Object(const Object& b);
 
+  Object() {}
   ~Object() { clear(); }
+  Object(const Object& b);
+  Object& operator=(const Object& b);
 
   // TODO: Move this out of the class namespace, call them
   // make_object_. 
@@ -133,7 +99,7 @@ public:
   // cases where we pass constant rvalues.
   void                clear();
 
-  type_type           type() const                            { return (type_type)(m_flags & mask_type); }
+  type_type           type() const                            { return static_cast<type_type>(m_flags & mask_type); }
   uint32_t            flags() const                           { return m_flags & mask_flags; }
 
   void                set_flags(uint32_t f)                   { m_flags |= f & mask_public; }
@@ -162,6 +128,7 @@ public:
   string_type&        as_string()                             { check_throw(TYPE_STRING); return _string(); }
   const string_type&  as_string() const                       { check_throw(TYPE_STRING); return _string(); }
   const string_type&  as_string_c() const                     { check_throw(TYPE_STRING); return _string(); }
+  const char*         as_c_str() const                        { check_throw(TYPE_STRING); return _string().c_str(); }
   list_type&          as_list()                               { check_throw(TYPE_LIST); return _list(); }
   const list_type&    as_list() const                         { check_throw(TYPE_LIST); return _list(); }
   map_type&           as_map()                                { check_throw(TYPE_MAP); return _map(); }
@@ -179,6 +146,8 @@ public:
   raw_map&            as_raw_map()                            { check_throw(TYPE_RAW_MAP); return _raw_map(); }
   const raw_map&      as_raw_map() const                      { check_throw(TYPE_RAW_MAP); return _raw_map(); }
 
+  template <typename T> T as_value_type(const char* err_msg) const { check_value_throw<T>(err_msg); return _value(); }
+
   bool                has_key(const key_type& k) const        { check_throw(TYPE_MAP); return _map().find(k) != _map().end(); }
   bool                has_key_value(const key_type& k) const  { check_throw(TYPE_MAP); return check(_map().find(k), TYPE_VALUE); }
   bool                has_key_string(const key_type& k) const { check_throw(TYPE_MAP); return check(_map().find(k), TYPE_STRING); }
@@ -227,9 +196,9 @@ public:
   Object&             insert_back(const Object& b)                   { check_throw(TYPE_LIST); return *_list().insert(_list().end(), b); }
 
   // Copy and merge operations:
-  Object&             move(Object& b);
-  Object&             swap(Object& b);
-  Object&             swap_same_type(Object& b);
+  Object&             move(Object& b) noexcept;
+  Object&             swap(Object& b) noexcept;
+  Object&             swap_same_type(Object& b) noexcept;
 
   // Only map entries are merged.
   Object&             merge_move(Object& object, uint32_t maxDepth = ~uint32_t());
@@ -237,16 +206,67 @@ public:
                                  uint32_t skip_mask = flag_static_data,
                                  uint32_t maxDepth = ~uint32_t());
 
-  Object&             operator = (const Object& b);
-
   // Internal:
-  void                swap_same_type(Object& left, Object& right);
+  static void         swap_same_type(Object& left, Object& right) noexcept;
 
  private:
-  inline bool         check(map_type::const_iterator itr, type_type t) const { return itr != _map().end() && itr->second.type() == t; }
-  inline void         check_throw(type_type t) const                         { if (t != type()) throw bencode_error("Wrong object type."); }
+  bool                check(map_type::const_iterator itr, type_type t) const { return itr != _map().end() && itr->second.type() == t; }
+  void                check_throw(type_type t) const                         { if (t != type()) throw bencode_error("Wrong object type."); }
 
-  uint32_t            m_flags;
+  template <typename T> void check_value_throw(const char* err_msg) const;
+
+  uint32_t            m_flags{TYPE_NONE};
+
+#ifndef HAVE_STDCXX_0X
+  value_type&         _value()             { return t_value; }
+  const value_type&   _value() const       { return t_value; }
+  string_type&        _string()            { return t_string; }
+  const string_type&  _string() const      { return t_string; }
+  list_type&          _list()              { return t_list; }
+  const list_type&    _list() const        { return t_list; }
+  map_type&           _map()               { return *t_map; }
+  const map_type&     _map() const         { return *t_map; }
+  map_ptr_type&       _map_ptr()           { return t_map; }
+  const map_ptr_type& _map_ptr() const     { return t_map; }
+  dict_key_type&       _dict_key()         { return t_dict_key; }
+  const dict_key_type& _dict_key() const   { return t_dict_key; }
+  raw_object&         _raw_object()        { return t_raw_object; }
+  const raw_object&   _raw_object() const  { return t_raw_object; }
+  raw_bencode&        _raw_bencode()       { return t_raw_bencode; }
+  const raw_bencode&  _raw_bencode() const { return t_raw_bencode; }
+  raw_string&         _raw_string()        { return t_raw_string; }
+  const raw_string&   _raw_string() const  { return t_raw_string; }
+  raw_list&           _raw_list()          { return t_raw_list; }
+  const raw_list&     _raw_list() const    { return t_raw_list; }
+  raw_map&            _raw_map()           { return t_raw_map; }
+  const raw_map&      _raw_map() const     { return t_raw_map; }
+
+  union pod_types {
+    value_type    t_value;
+    raw_object    t_raw_object;
+    raw_bencode   t_raw_bencode;
+    raw_string    t_raw_string;
+    raw_list      t_raw_list;
+    raw_map       t_raw_map;
+  };
+
+  union {
+    pod_types     t_pod;
+
+    value_type    t_value;
+    string_type   t_string;
+    list_type     t_list;
+    map_type*     t_map;
+    dict_key_type t_dict_key;
+    raw_object    t_raw_object;
+    raw_bencode   t_raw_bencode;
+    raw_string    t_raw_string;
+    raw_list      t_raw_list;
+    raw_map       t_raw_map;
+  };
+
+#else
+  // #error "WTF we're testing C++11 now."
 
   value_type&         _value()             { return reinterpret_cast<value_type&>(t_pod); }
   const value_type&   _value() const       { return reinterpret_cast<const value_type&>(t_pod); }
@@ -283,17 +303,18 @@ public:
     char      t_list[sizeof(list_type)];
     char      t_dict_key[sizeof(dict_key_type)];
   };
+#endif
 };
 
 inline
-Object::Object(const Object& b) {
-  m_flags = b.m_flags & (mask_type | mask_public);
+Object::Object(const Object& b) :
+    m_flags(b.m_flags & (mask_type | mask_public)) {
 
   switch (type()) {
   case TYPE_NONE:
   case TYPE_RAW_BENCODE:
-  case TYPE_RAW_STRING: 
-  case TYPE_RAW_LIST: 
+  case TYPE_RAW_STRING:
+  case TYPE_RAW_LIST:
   case TYPE_RAW_MAP:
   case TYPE_VALUE:       t_pod = b.t_pod; break;
   case TYPE_STRING:      new (&_string()) string_type(b._string()); break;
@@ -404,7 +425,7 @@ Object::clear() {
 }
 
 inline void
-Object::swap_same_type(Object& left, Object& right) {
+Object::swap_same_type(Object& left, Object& right) noexcept {
   std::swap(left.m_flags, right.m_flags);
 
   switch (left.type()) {
@@ -417,7 +438,7 @@ Object::swap_same_type(Object& left, Object& right) {
   }
 }
 
-inline void swap(Object& left, Object& right) { left.swap(right); }
+inline void swap(Object& left, Object& right) noexcept { left.swap(right); }
 
 inline bool
 object_equal(const Object& left, const Object& right) {
@@ -432,6 +453,19 @@ object_equal(const Object& left, const Object& right) {
   }
 }
 
+template <typename T>
+inline void
+Object::check_value_throw(const char* err_msg) const {
+  if (!std::numeric_limits<T>::is_integer)
+    throw internal_error("Tried to check value with non-integer type.");
+
+  if (!is_value())
+    throw bencode_error(err_msg);
+
+  if (!(_value() >= std::numeric_limits<T>::min() && _value() <= std::numeric_limits<T>::max()))
+    throw bencode_error(err_msg);
 }
 
+} // namespace torrent
+
 #endif
