@@ -2,20 +2,18 @@
 
 #include "directory_events.h"
 
-#include <string>
-#include <errno.h>
-#include <unistd.h>
 #include <algorithm>
+#include <string>
+#include <unistd.h>
 
 #ifdef USE_INOTIFY
 #include <sys/inotify.h>
 #endif
 
-#include "rak/error_number.h"
 #include "net/socket_fd.h"
+#include "rak/error_number.h"
 #include "torrent/exceptions.h"
 #include "torrent/poll.h"
-#include "torrent/utils/thread.h"
 
 namespace torrent {
 
@@ -24,7 +22,7 @@ directory_events::open() {
   if (m_fileDesc != -1)
     return true;
 
-  rak::error_number::current().clear_global();
+  rak::error_number::clear_global();
 
 #ifdef USE_INOTIFY
   m_fileDesc = inotify_init();
@@ -89,10 +87,10 @@ directory_events::notify_on(const std::string& path, [[maybe_unused]] int flags,
   if (result == -1)
     throw input_error("Call to inotify_add_watch(...) failed: " + std::string(rak::error_number::current().c_str()));
 
-  auto itr = m_wd_list.insert(m_wd_list.end(), watch_descriptor());
-  itr->descriptor = result;
-  itr->path = path + (*path.rbegin() != '/' ? "/" : "");
-  itr->slot = slot;
+  auto& wd = m_wd_list.emplace_back();
+  wd.descriptor = result;
+  wd.path = path + (path.back() != '/' ? "/" : "");
+  wd.slot = slot;
 
 #else
   throw input_error("No support for inotify.");
@@ -139,4 +137,4 @@ void
 directory_events::event_error() {
 }
 
-}
+} // namespace torrent
