@@ -35,7 +35,6 @@ public:
   void                send_scrape() override;
 
   void                close() override;
-  void                disown() override;
 
   tracker_enum        type() const override;
 
@@ -59,7 +58,7 @@ private:
   bool                process_announce_output();
   bool                process_error_output();
 
-  bool                parse_udp_url(const std::string& url, hostname_type& hostname, int& port) const;
+  static bool         parse_udp_url(const std::string& url, hostname_type& hostname, int& port);
 
   bool                m_resolver_requesting{false};
   bool                m_sending_announce{false};
@@ -85,6 +84,6 @@ private:
   std::chrono::microseconds m_time_last_resolved{};
 };
 
-}
+} // namespace torrent
 
 #endif
