@@ -1,6 +1,7 @@
 #ifndef LIBTORRENT_UTILS_FUNCTIONAL_H
 #define LIBTORRENT_UTILS_FUNCTIONAL_H
 
+#include <tuple>
 #include <utility>
 
 namespace utils {
@@ -15,12 +16,14 @@ slot_list_call(const Container& slot_list, Args&&... args) {
   auto first = slot_list.begin();
   auto next  = slot_list.begin();
 
+  auto args_tuple = std::forward_as_tuple(std::forward<Args>(args)...);
+
   while (++next != slot_list.end()) {
-    (*first)(std::forward<Args>(args)...);
+    std::apply(*first, args_tuple);
     first = next;
   }
 
-  (*first)(std::forward<Args>(args)...);
+  std::apply(*first, args_tuple);
 }
 
 } // namespace utils
