@@ -10,16 +10,16 @@ ExceptionProtector::protect(const CppUnit::Functor& functor, const CppUnit::Prot
   try {
     return functor();
 
-  } catch (CppUnit::Exception &failure) {
+  } catch (const CppUnit::Exception& failure) {
     reportFailure( context, failure );
 
-  } catch (torrent::base_error& e) {
+  } catch (const torrent::base_error& e) {
     std::string short_description("uncaught exception of base type torrent::base_error: " + std::string(typeid(e).name()));
 
     CppUnit::Message message(short_description, e.what());
     reportError(context, message);
 
-  } catch ( std::exception &e ) {
+  } catch (const std::exception& e) {
     std::string short_description("uncaught exception of type ");
 
     short_description += CppUnit::TypeInfoHelper::getClassName(typeid(e));
@@ -27,7 +27,7 @@ ExceptionProtector::protect(const CppUnit::Functor& functor, const CppUnit::Prot
     CppUnit::Message message(short_description, e.what());
     reportError(context, message);
 
-  } catch ( ... ) {
+  } catch (...) {
     reportError(context, CppUnit::Message("uncaught exception of unknown type"));
   }
 
