@@ -2,8 +2,8 @@
 
 #include <cstdlib>
 #include <stdexcept>
-#include <signal.h>
-#include <string.h>
+#include <csignal>
+#include <cstring>
 #include <cppunit/BriefTestProgressListener.h>
 #include <cppunit/CompilerOutputter.h>
 #include <cppunit/TestResult.h>
@@ -27,6 +27,7 @@ CPPUNIT_REGISTRY_ADD_TO_DEFAULT("data");
 CPPUNIT_REGISTRY_ADD_TO_DEFAULT("net");
 CPPUNIT_REGISTRY_ADD_TO_DEFAULT("tracker");
 
+namespace {
 void
 do_test_panic(int signum) {
   signal(signum, SIG_DFL);
@@ -64,6 +65,7 @@ register_signal_handlers() {
     exit(-1);
   }
 }
+} // namespace
 
 int main([[maybe_unused]] int argc, [[maybe_unused]] char* argv[]) {
   register_signal_handlers();
@@ -92,7 +94,7 @@ int main([[maybe_unused]] int argc, [[maybe_unused]] char* argv[]) {
     CppUnit::CompilerOutputter outputter( &result, std::cerr );
     outputter.write();
 
-  } catch ( std::invalid_argument &e ) { // Test path not resolved
+  } catch (const std::invalid_argument& e) { // Test path not resolved
     std::cerr  <<  std::endl <<  "ERROR: "  <<  e.what() << std::endl;
     return 1;
   }
