@@ -1,6 +1,6 @@
 #include "config.h"
 
-#include <tr1/functional>
+#include <functional>
 #include <torrent/exceptions.h>
 #include <torrent/utils/directory_events.h>
 
@@ -8,8 +8,6 @@
 
 CPPUNIT_TEST_SUITE_REGISTRATION(utils_directory_events_test);
 
-namespace tr1 { using namespace std::tr1; }
-
 void
 utils_directory_events_test::setUp() {
 }
