@@ -14,15 +14,11 @@
 
 CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(test_log, "torrent/utils");
 
-namespace torrent {
-typedef std::vector<std::pair<std::string, log_slot> > log_output_list;
-extern log_output_list log_outputs;
-}
-
-const char* expected_output = NULL;
+namespace {
+const char* expected_output = nullptr;
 unsigned int output_mask;
 
-static void
+void
 test_output(const char* output, unsigned int length, unsigned int mask) {
   CPPUNIT_ASSERT_MESSAGE("'" + std::string(output) + "' != '" + std::string(expected_output) + "'",
                          std::strcmp(output, expected_output) == 0);
@@ -30,6 +26,13 @@ test_output(const char* output, unsigned int length, unsigned int mask) {
   output_mask |= mask;
 }
 
+} // namespace
+
+namespace torrent {
+typedef std::vector<std::pair<std::string, log_slot> > log_output_list;
+extern log_output_list log_outputs;
+}
+
 #define LTUNIT_ASSERT_OUTPUT(group, mask, expected, ...) \
   output_mask = 0; expected_output = expected;           \
   lt_log_print(group, __VA_ARGS__);                      \
@@ -76,7 +79,7 @@ test_log::test_output_open() {
 
   // try {
   //   torrent::log_open_output("test_output_1", torrent::log_slot());
-  // } catch (torrent::input_error& e) {
+  // } catch (const torrent::input_error&) {
   //   return;
   // }
 
